@@ -32,17 +32,21 @@
 #include <libmissioncontrol/mc-account.h>
 
 #include <libempathy/empathy-contact-manager.h>
+#include <libempathy/empathy-utils.h>
 
 #include "empathy-contact-widget.h"
-#include "gossip-account-chooser.h"
-#include "gossip-ui-utils.h"
+#include "empathy-account-chooser.h"
+#include "empathy-ui-utils.h"
+
+/* Delay before updating the widget when the id entry changed (ms) */
+#define ID_CHANGED_TIMEOUT 500
 
 typedef struct {
-	GossipContact   *contact;
+	EmpathyContact  *contact;
 	gboolean         is_user;
-	gboolean         editable;
-	gboolean         can_change_contact;
+	EmpathyContactWidgetType type;
 	GtkCellRenderer *renderer;
+	guint            widget_id_timeout;
 
 	GtkWidget       *vbox_contact_widget;
 
@@ -89,7 +93,7 @@ static void     contact_widget_destroy_cb                 (GtkWidget
 							   EmpathyContactWidget  *information);
 static void     contact_widget_remove_contact             (EmpathyContactWidget  *information);
 static void     contact_widget_set_contact                (EmpathyContactWidget  *information,
-							   GossipContact         *contact);
+							   EmpathyContact         *contact);
 static void     contact_widget_contact_setup              (EmpathyContactWidget  *information);
 static void     contact_widget_contact_update             (EmpathyContactWidget  *information);
 static gboolean contact_widget_update_contact             (EmpathyContactWidget  *information);
@@ -139,26 +143,21 @@ enum {
 };
 
 GtkWidget *
-empathy_contact_widget_new (GossipContact *contact,
-			    gboolean       editable)
+empathy_contact_widget_new (EmpathyContact           *contact,
+			    EmpathyContactWidgetType  type)
 {
 	EmpathyContactWidget *information;
 	GladeXML             *glade;
 
 	information = g_slice_new0 (EmpathyContactWidget);
-	information->editable = editable;
+	information->type = type;
 	if (contact) {
-		GossipContact *user_contact;
-	
-		user_contact = gossip_contact_get_user (contact);
-		information->is_user = gossip_contact_equal (contact, user_contact);
-		information->can_change_contact = FALSE;
+		information->is_user = empathy_contact_is_user (contact);
 	} else {
 		information->is_user = FALSE;
-		information->can_change_contact = TRUE;
 	}
 
-	glade = gossip_glade_get_file ("empathy-contact-widget.glade",
+	glade = empathy_glade_get_file ("empathy-contact-widget.glade",
 				       "vbox_contact_widget",
 				       NULL,
 				       "vbox_contact_widget", &information->vbox_contact_widget,
@@ -181,7 +180,7 @@ empathy_contact_widget_new (GossipContact *contact,
 				       "hbox_client_requested", &information->hbow_client_requested,
 				       NULL);
 
-	gossip_glade_connect (glade,
+	empathy_glade_connect (glade,
 			      information,
 			      "vbox_contact_widget", "destroy", contact_widget_destroy_cb,
 			      "entry_group", "changed", contact_widget_entry_group_changed_cb,
@@ -208,7 +207,7 @@ empathy_contact_widget_new (GossipContact *contact,
 	return information->vbox_contact_widget;
 }
 
-GossipContact *
+EmpathyContact *
 empathy_contact_widget_get_contact (GtkWidget *widget)
 {
 	EmpathyContactWidget *information;
@@ -228,6 +227,11 @@ contact_widget_destroy_cb (GtkWidget            *widget,
 			   EmpathyContactWidget *information)
 {
 	contact_widget_remove_contact (information);
+
+	if (information->widget_id_timeout != 0) {
+		g_source_remove (information->widget_id_timeout);
+	}
+
 	g_slice_free (EmpathyContactWidget, information);
 }
 
@@ -255,7 +259,7 @@ contact_widget_remove_contact (EmpathyContactWidget *information)
 
 static void
 contact_widget_set_contact (EmpathyContactWidget *information,
-			    GossipContact        *contact)
+			    EmpathyContact        *contact)
 {
 	contact_widget_remove_contact (information);
 	if (contact) {
@@ -269,10 +273,63 @@ contact_widget_set_contact (EmpathyContactWidget *information,
 	contact_widget_client_update (information);
 }
 
+static gboolean
+contact_widget_can_add_contact_to_account (McAccount *account,
+					   gpointer   user_data)
+{
+	MissionControl *mc;
+	TpConn         *tp_conn;
+	McProfile      *profile;
+	const gchar    *protocol_name;
+
+	mc = empathy_mission_control_new ();
+	tp_conn = mission_control_get_connection (mc, account, NULL);
+	g_object_unref (mc);
+	if (tp_conn == NULL) {
+		/* Account is disconnected */
+		return FALSE;
+	}
+	g_object_unref (tp_conn);
+
+	profile = mc_account_get_profile (account);
+	protocol_name = mc_profile_get_protocol_name (profile);
+	if (strcmp (protocol_name, "local-xmpp") == 0) {
+		/* We can't add accounts to a XMPP LL connection
+		 * FIXME: We should inspect the flags of the contact list group interface
+		 */
+		g_object_unref (profile);
+		return FALSE;
+	}
+
+	g_object_unref (profile);
+	return TRUE;
+}
+
+static gboolean
+contact_widget_id_activate_timeout (EmpathyContactWidget *self)
+{
+	contact_widget_update_contact (self);
+	return FALSE;
+}
+
+static void
+contact_widget_id_changed_cb (GtkEntry             *entry,
+                              EmpathyContactWidget *self)
+{
+	if (self->widget_id_timeout != 0) {		
+		g_source_remove (self->widget_id_timeout);
+	}
+
+	self->widget_id_timeout =
+		g_timeout_add (ID_CHANGED_TIMEOUT,
+			       (GSourceFunc) contact_widget_id_activate_timeout,
+			       self);
+}
+
 static void
 contact_widget_contact_setup (EmpathyContactWidget *information)
 {
-	/* FIXME: Use GossipAvatarImage if (editable && is_user)  */
+	/* FIXME: Use EmpathyAvatarImage if (editable && is_user)  */
 	information->widget_avatar = gtk_image_new ();
 	gtk_box_pack_end (GTK_BOX (information->hbox_contact),
 	 		  information->widget_avatar,
@@ -280,14 +337,20 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
 	 		  6);
 
 	/* Setup account label/chooser */
-	if (information->can_change_contact) {
-		information->widget_account = gossip_account_chooser_new ();
+	if (information->type == CONTACT_WIDGET_TYPE_ADD) {
+		information->widget_account = empathy_account_chooser_new ();
+		empathy_account_chooser_set_filter (
+			EMPATHY_ACCOUNT_CHOOSER (information->widget_account),
+			contact_widget_can_add_contact_to_account,
+			NULL);
+
 		g_signal_connect (information->widget_account, "changed",
 				  G_CALLBACK (contact_widget_account_changed_cb),
 				  information);
 	} else {
 		information->widget_account = gtk_label_new (NULL);
 		gtk_label_set_selectable (GTK_LABEL (information->widget_account), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (information->widget_account), 0, 0.5);
 	}
 	gtk_table_attach_defaults (GTK_TABLE (information->table_contact),
 				   information->widget_account,
@@ -295,14 +358,18 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
 	gtk_widget_show (information->widget_account);
 
 	/* Setup id label/entry */
-	if (information->can_change_contact) {
+	if (information->type == CONTACT_WIDGET_TYPE_ADD) {
 		information->widget_id = gtk_entry_new ();
 		g_signal_connect (information->widget_id, "focus-out-event",
 				  G_CALLBACK (contact_widget_id_focus_out_cb),
 				  information);
+		g_signal_connect (information->widget_id, "changed",
+				  G_CALLBACK (contact_widget_id_changed_cb),
+				  information);
 	} else {
 		information->widget_id = gtk_label_new (NULL);
 		gtk_label_set_selectable (GTK_LABEL (information->widget_id), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (information->widget_id), 0, 0.5);
 	}
 	gtk_table_attach_defaults (GTK_TABLE (information->table_contact),
 				   information->widget_id,
@@ -310,7 +377,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
 	gtk_widget_show (information->widget_id);
 
 	/* Setup alias label/entry */
-	if (information->editable) {
+	if (information->type > CONTACT_WIDGET_TYPE_SHOW) {
 		information->widget_alias = gtk_entry_new ();
 		g_signal_connect (information->widget_alias, "focus-out-event",
 				  G_CALLBACK (contact_widget_entry_alias_focus_event_cb),
@@ -318,6 +385,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
 	} else {
 		information->widget_alias = gtk_label_new (NULL);
 		gtk_label_set_selectable (GTK_LABEL (information->widget_alias), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (information->widget_alias), 0, 0.5);
 	}
 	gtk_table_attach_defaults (GTK_TABLE (information->table_contact),
 				   information->widget_alias,
@@ -343,36 +411,32 @@ contact_widget_contact_update (EmpathyContactWidget *information)
 					  G_CALLBACK (contact_widget_avatar_notify_cb),
 					  information);
 
-		account = gossip_contact_get_account (information->contact);
-		id = gossip_contact_get_id (information->contact);
+		account = empathy_contact_get_account (information->contact);
+		id = empathy_contact_get_id (information->contact);
 	}
 
 	/* Update account widget */
-	if (information->can_change_contact) {
+	if (information->type == CONTACT_WIDGET_TYPE_ADD) {
 		if (account) {
 			g_signal_handlers_block_by_func (information->widget_account,
 							 contact_widget_account_changed_cb,
 							 information);
-			gossip_account_chooser_set_account (GOSSIP_ACCOUNT_CHOOSER (information->widget_account),
+			empathy_account_chooser_set_account (EMPATHY_ACCOUNT_CHOOSER (information->widget_account),
 							    account);
 			g_signal_handlers_unblock_by_func (information->widget_account,
 							   contact_widget_account_changed_cb,
 							   information);
 		}
-	}
-	else if (account) {
-		const gchar *name;
-
-		name = mc_account_get_display_name (account);
-		gtk_label_set_label (GTK_LABEL (information->widget_account), name);
-	}
-
-	/* Update id widget */
-	if (information->can_change_contact) {
 		if (!G_STR_EMPTY (id)) {
 			gtk_entry_set_text (GTK_ENTRY (information->widget_id), id);
 		}
 	} else {
+		if (account) {
+			const gchar *name;
+
+			name = mc_account_get_display_name (account);
+			gtk_label_set_label (GTK_LABEL (information->widget_account), name);
+		}
 		gtk_label_set_label (GTK_LABEL (information->widget_id), id);
 	}
 
@@ -399,12 +463,12 @@ contact_widget_update_contact (EmpathyContactWidget *information)
 	McAccount   *account;
 	const gchar *id;
 
-	account = gossip_account_chooser_get_account (GOSSIP_ACCOUNT_CHOOSER (information->widget_account));
+	account = empathy_account_chooser_get_account (EMPATHY_ACCOUNT_CHOOSER (information->widget_account));
 	id = gtk_entry_get_text (GTK_ENTRY (information->widget_id));
 
 	if (account && !G_STR_EMPTY (id)) {
 		EmpathyContactManager *manager;
-		GossipContact         *contact;
+		EmpathyContact         *contact;
 
 		manager = empathy_contact_manager_new ();
 		contact = empathy_contact_manager_create (manager, account, id);
@@ -444,7 +508,7 @@ contact_widget_entry_alias_focus_event_cb (GtkEditable          *editable,
 		const gchar *name;
 
 		name = gtk_entry_get_text (GTK_ENTRY (editable));
-		gossip_contact_set_name (information->contact, name);
+		empathy_contact_set_name (information->contact, name);
 	}
 
 	return FALSE;
@@ -453,12 +517,12 @@ contact_widget_entry_alias_focus_event_cb (GtkEditable          *editable,
 static void
 contact_widget_name_notify_cb (EmpathyContactWidget *information)
 {
-	if (information->editable) {
+	if (GTK_IS_ENTRY (information->widget_alias)) {
 		gtk_entry_set_text (GTK_ENTRY (information->widget_alias),
-				    gossip_contact_get_name (information->contact));
+				    empathy_contact_get_name (information->contact));
 	} else {
 		gtk_label_set_label (GTK_LABEL (information->widget_alias),
-				     gossip_contact_get_name (information->contact));
+				     empathy_contact_get_name (information->contact));
 	}
 }
 
@@ -466,9 +530,9 @@ static void
 contact_widget_presence_notify_cb (EmpathyContactWidget *information)
 {
 	gtk_label_set_text (GTK_LABEL (information->label_status),
-			    gossip_contact_get_status (information->contact));
+			    empathy_contact_get_status (information->contact));
 	gtk_image_set_from_icon_name (GTK_IMAGE (information->image_state),
-				      gossip_icon_name_for_contact (information->contact),
+				      empathy_icon_name_for_contact (information->contact),
 				      GTK_ICON_SIZE_BUTTON);
 
 }
@@ -478,7 +542,7 @@ contact_widget_avatar_notify_cb (EmpathyContactWidget *information)
 {
 	GdkPixbuf *avatar_pixbuf;
 
-	avatar_pixbuf = gossip_pixbuf_avatar_from_contact_scaled (information->contact,
+	avatar_pixbuf = empathy_pixbuf_avatar_from_contact_scaled (information->contact,
 								  48, 48);
 
 	if (avatar_pixbuf) {
@@ -494,7 +558,7 @@ contact_widget_avatar_notify_cb (EmpathyContactWidget *information)
 static void
 contact_widget_groups_setup (EmpathyContactWidget *information)
 {
-	if (information->editable) {
+	if (information->type > CONTACT_WIDGET_TYPE_SHOW) {
 		contact_widget_model_setup (information);
 	}
 }
@@ -502,7 +566,8 @@ contact_widget_groups_setup (EmpathyContactWidget *information)
 static void
 contact_widget_groups_update (EmpathyContactWidget *information)
 {
-	if (information->editable && information->contact) {
+	if (information->type > CONTACT_WIDGET_TYPE_SHOW &&
+	    information->contact) {
 		g_signal_connect_swapped (information->contact, "notify::groups",
 					  G_CALLBACK (contact_widget_groups_notify_cb),
 					  information);
@@ -605,7 +670,7 @@ contact_widget_groups_populate_data (EmpathyContactWidget *information)
 
 	manager = empathy_contact_manager_new ();
 	all_groups = empathy_contact_manager_get_groups (manager);
-	my_groups = gossip_contact_get_groups (information->contact);
+	my_groups = empathy_contact_get_groups (information->contact);
 	g_object_unref (manager);
 
 	for (l = all_groups; l; l = l->next) {
@@ -727,9 +792,9 @@ contact_widget_cell_toggled (GtkCellRendererToggle *cell,
 
 	if (group) {
 		if (enabled) {
-			gossip_contact_remove_group (information->contact, group);
+			empathy_contact_remove_group (information->contact, group);
 		} else {
-			gossip_contact_add_group (information->contact, group);	
+			empathy_contact_add_group (information->contact, group);	
 		}
 
 		g_free (group);
@@ -781,7 +846,7 @@ contact_widget_button_group_clicked_cb (GtkButton             *button,
 			    COL_ENABLED, TRUE,
 			    -1);
 
-	gossip_contact_add_group (information->contact, group);
+	empathy_contact_add_group (information->contact, group);
 }
 
 static void
